@@ -1,7 +1,7 @@
-%
+%%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2004-2020. All Rights Reserved.
+%% Copyright Ericsson AB 2004-2025. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -21,6 +21,121 @@
 %%
 
 -module(ssh).
+-moduledoc """
+Main API of the ssh application
+
+This is the interface module for the `SSH` application. The Secure Shell (SSH)
+Protocol is a protocol for secure remote login and other secure network services
+over an insecure network. See [ssh](ssh_app.md#supported) for details of
+supported RFCs, versions, algorithms and unicode handling.
+
+With the SSH application it is possible to start _clients_ and to start
+_daemons_ (servers).
+
+Clients are started with `connect/2`, `connect/3` or `connect/4`. They open an
+encrypted connection on top of TCP/IP. In that encrypted connection one or more
+channels could be opened with
+[ssh_connection:session_channel/2,4](`ssh_connection:session_channel/2`).
+
+Each channel is an isolated "pipe" between a client-side process and a
+server-side process. Those process pairs could handle for example file transfers
+(sftp) or remote command execution (shell, exec and/or cli). If a custom shell
+is implemented, the user of the client could execute the special commands
+remotely. Note that the user is not necessarily a human but probably a system
+interfacing the SSH app.
+
+A server-side subssystem (channel) server is requested by the client with
+`ssh_connection:subsystem/4`.
+
+A server (daemon) is started with [daemon/1](`daemon/2`), `daemon/2` or
+[daemon/3](`daemon/2`). Possible channel handlers (subsystems) are declared with
+the [subsystem](`t:subsystem_daemon_option/0`) option when the daemon is
+started.
+
+To just run a shell on a remote machine, there are functions that bundles the
+needed three steps needed into one: [shell/1,2,3](`shell/1`). Similarly, to just
+open an sftp (file transfer) connection to a remote machine, the simplest way is
+to use [ssh_sftp:start_channel/1,2,3](`ssh_sftp:start_channel/1`).
+
+To write your own client channel handler, use the behaviour
+`m:ssh_client_channel`. For server channel handlers use `m:ssh_server_channel`
+behaviour (replaces ssh_daemon_channel).
+
+Both clients and daemons accept options that control the exact behaviour. Some
+options are common to both. The three sets are called
+[Client Options](`t:client_options/0`), [Daemon Options](`t:daemon_options/0`)
+and [Common Options](`t:common_options/0`).
+
+The descriptions of the options uses the
+[Erlang Type Language](`e:system:typespec.md`) with explaining text.
+
+> #### Note {: .info }
+>
+> See also [SSH Application Reference](index.html) and [Examples](using_ssh.md) section.
+
+## Keys and files
+
+A number of objects must be present for the SSH application to work. Those
+objects are per default stored in files. The default names, paths and file
+formats are the same as for [OpenSSH](http://www.openssh.com). Keys could be
+generated with the `ssh-keygen` program from OpenSSH. See the
+[User's Guide](using_ssh.md#running-an-erlang-ssh-daemon).
+
+The paths could easily be changed by options:
+[`user_dir`](`t:ssh_file:user_dir_common_option/0`) and
+[`system_dir`](`t:ssh_file:system_dir_daemon_option/0`).
+
+A completely different storage could be interfaced by writing callback modules
+using the behaviours `m:ssh_client_key_api` and/or `m:ssh_server_key_api`. A
+callback module is installed with the option
+[`key_cb`](`t:key_cb_common_option/0`) to the client and/or the daemon.
+
+### Daemons
+
+The keys are by default stored in files:
+
+- Mandatory: one or more _Host key(s)_, both private and public. Default is to
+  store them in the directory `/etc/ssh` in the files
+
+  - `ssh_host_dsa_key` and `ssh_host_dsa_key.pub`
+  - `ssh_host_rsa_key` and `ssh_host_rsa_key.pub`
+  - `ssh_host_ecdsa_key` and `ssh_host_ecdsa_key.pub`
+
+  The host keys directory could be changed with the option
+  [`system_dir`](`t:ssh_file:system_dir_daemon_option/0`).
+
+- Optional: one or more _User's public key_ in case of `publickey`
+  authorization. Default is to store them concatenated in the file
+  `.ssh/authorized_keys` in the user's home directory.
+
+  The user keys directory could be changed with the option
+  [`user_dir`](`t:ssh_file:user_dir_common_option/0`).
+
+### Clients
+
+The keys and some other data are by default stored in files in the directory
+`.ssh` in the user's home directory.
+
+The directory could be changed with the option
+[`user_dir`](`t:ssh_file:user_dir_common_option/0`).
+
+- Optional: a list of _Host public key(s)_ for previously connected hosts. This
+  list is handled by the SSH application without any need of user assistance.
+  The default is to store them in the file `known_hosts`.
+
+  The `t:host_accepting_client_options/0` are associated with this list of keys.
+
+- Optional: one or more _User's private key(s)_ in case of `publickey`
+  authorization. The default files are
+  - `id_dsa` and `id_dsa.pub`
+  - `id_rsa` and `id_rsa.pub`
+  - `id_ecdsa` and `id_ecdsa.pub`
+""".
+-moduledoc(#{titles =>
+                 [{type,<<"Client Options">>},
+                  {type,<<"Daemon Options">>},
+                  {type,<<"Common Options">>},
+                  {type,<<"Other data types">>}]}).
 
 -include("ssh.hrl").
 -include("ssh_connect.hrl").
@@ -35,6 +150,7 @@
 	 channel_info/3,
 	 daemon/1, daemon/2, daemon/3,
 	 daemon_info/1, daemon_info/2,
+         daemon_replace_options/2,
          set_sock_opts/2, get_sock_opts/2,
 	 default_algorithms/0,
          chk_algos_opts/1,
@@ -87,19 +203,46 @@
 	     ]).
 
 
+-doc """
+Opaque data type representing a daemon.
+
+Returned by the functions [`daemon/1,2,3`](`daemon/1`).
+""".
+-doc(#{title => <<"Other data types">>}).
 -opaque daemon_ref()         :: pid() .
+-doc """
+Opaque data type representing a channel inside a connection.
+
+Returned by the functions
+[ssh_connection:session_channel/2,4](`ssh_connection:session_channel/2`).
+""".
+-doc(#{title => <<"Other data types">>}).
 -opaque channel_id()     :: non_neg_integer().
+-doc """
+Opaque data type representing a connection between a client and a server
+(daemon).
+
+Returned by the functions [`connect/2,3,4`](`connect/3`) and
+[`ssh_sftp:start_channel/2,3`](`ssh_sftp:start_channel/2`).
+""".
+-doc(#{title => <<"Other data types">>}).
 -type connection_ref()       :: pid().  % should be -opaque, but that gives problems
 
 %%--------------------------------------------------------------------
 %% Description: Starts the ssh application. Default type
 %% is temporary. see application(3)
 %%--------------------------------------------------------------------
+-doc(#{equiv => start/1}).
 -spec start() -> ok | {error, term()}.
 
 start() ->
     start(temporary).
 
+-doc """
+Utility function that starts the applications `crypto`, `public_key`, and `ssh`.
+Default type is `temporary`. For more information, see the `m:application`
+manual page in Kernel.
+""".
 -spec start(Type) -> ok | {error, term()} when
       Type :: permanent | transient | temporary .
 
@@ -108,7 +251,7 @@ start(Type) ->
         {ok, _} ->
             %% Clear cached default_algorithms (if exists) ...
             ssh_transport:clear_default_algorithms_env(),
-            %% ... and rebuld them taking configure options in account
+            %% ... and rebuild them taking configure options in account
             ssh_transport:default_algorithms(),
             ok;
         Other ->
@@ -118,6 +261,10 @@ start(Type) ->
 %%--------------------------------------------------------------------
 %% Description: Stops the ssh application.
 %%--------------------------------------------------------------------
+-doc """
+Stops the `ssh` application. For more information, see the `m:application`
+manual page in Kernel.
+""".
 -spec stop() -> ok | {error, term()}.
 
 stop() ->
@@ -126,25 +273,39 @@ stop() ->
 %%--------------------------------------------------------------------
 %% Description: Starts an ssh connection.
 %%--------------------------------------------------------------------
--spec connect(OpenTcpSocket, Options) -> {ok,connection_ref()} | {error,term()} when
+-define(IS_VALID_OPTIONS(Options), is_list(Options)).
+-define(IS_VALID_PORT(Port), (is_integer(Port) andalso Port > 0)).
+-define(IS_VALID_TIMEOUT(Timeout),
+        (Timeout == infinity
+         orelse (is_integer(Timeout)
+                 andalso Timeout >= 0))).
+
+-doc(#{equiv => connect/4}).
+-doc(#{since => <<"OTP 19.0">>}).
+-spec connect(OpenTcpSocket, Options)
+             -> {ok, connection_ref()}
+              | {error, term()} when
       OpenTcpSocket :: open_socket(),
       Options :: client_options().
 
-connect(OpenTcpSocket, Options) when is_list(Options) ->
-    connect(OpenTcpSocket, Options, infinity).
-
+connect(OpenTcpSocket, Options) when ?IS_VALID_OPTIONS(Options) ->
+    connect(OpenTcpSocket, Options, infinity);
+connect(_OpenTcpSocket, Options) ->
+    bad_arg([{options, Options}]).
 
+-doc(#{equiv => connect/4}).
 -spec connect(open_socket(), client_options(), timeout()) ->
-                     {ok,connection_ref()} | {error,term()}
+                     {ok, connection_ref()} | {error, term()}
            ; (host(), inet:port_number(), client_options()) ->
-                     {ok,connection_ref()} | {error,term()}.
+                     {ok, connection_ref()} | {error, term()}.
 
-connect(Host, Port, Options) when is_integer(Port),
-                                  Port>0,
-                                  is_list(Options) ->
-    connect(Host, Port, Options, infinity);
-
-connect(Socket, UserOptions, NegotiationTimeout) when is_list(UserOptions) ->
+connect(Host, Port, Options) when ?IS_VALID_PORT(Port),
+                                  ?IS_VALID_OPTIONS(Options) ->
+    Timeout = proplists:get_value(connect_timeout, Options, infinity),
+    connect(Host, Port, Options, Timeout);
+connect(Socket, UserOptions, NegotiationTimeout)
+  when ?IS_VALID_OPTIONS(UserOptions),
+       ?IS_VALID_TIMEOUT(NegotiationTimeout) ->
     case ssh_options:handle_options(client, UserOptions) of
 	{error, Error} ->
 	    {error, Error};
@@ -156,26 +317,45 @@ connect(Socket, UserOptions, NegotiationTimeout) when is_list(UserOptions) ->
                 {error,SockError} ->
                     {error,SockError}
             end
-    end.
+    end;
+connect(_HostOrSocket, PortOrOptions, OptionsOrTimeout) ->
+    bad_arg(PortOrOptions, OptionsOrTimeout).
 
+-doc """
+Connects to an SSH server at the `Host` on `Port`.
 
--spec connect(Host, Port, Options, NegotiationTimeout) -> {ok,connection_ref()} | {error,term()} when
+As an alternative, an already open TCP socket could be passed to the function in
+`TcpSocket`. The SSH initiation and negotiation will be initiated on that one
+with the SSH that should be at the other end.
+
+No channel is started. This is done by calling
+[ssh_connection:session_channel/2,4](`ssh_connection:session_channel/2`).
+
+The `NegotiationTimeout` is in milli-seconds. The default value is `infinity` or
+the value of the [`connect_timeout`](`t:connect_timeout_client_option/0`)
+option, if present. For connection timeout, use the option
+[`connect_timeout`](`t:connect_timeout_client_option/0`).
+""".
+-spec connect(Host, Port, Options, NegotiationTimeout)
+             -> {ok, connection_ref()}
+              | {error, term()} when
       Host :: host(),
       Port :: inet:port_number(),
       Options :: client_options(),
       NegotiationTimeout :: timeout().
 
-connect(Host0, Port, UserOptions, NegotiationTimeout) when is_integer(Port),
-                                                           Port>0,
-                                                           is_list(UserOptions) ->
+connect(Host0, Port, UserOptions, NegotiationTimeout)
+  when ?IS_VALID_PORT(Port),
+       ?IS_VALID_OPTIONS(UserOptions),
+       ?IS_VALID_TIMEOUT(NegotiationTimeout) ->
     case ssh_options:handle_options(client, UserOptions) of
 	{error, Reason} ->
             {error, Reason};
-        
+
         Options ->
-            SocketOpts = [{active,false} | ?GET_OPT(socket_options,Options)],
+            SocketOpts = ?GET_OPT(socket_options,Options) ++ [{active,false}],
             Host = mangle_connect_address(Host0, Options),
-            try 
+            try
                 transport_connect(Host, Port, SocketOpts, Options)
             of
                 {ok, Socket} ->
@@ -188,8 +368,42 @@ connect(Host0, Port, UserOptions, NegotiationTimeout) when is_integer(Port),
                 error:Error -> {error,Error};
                 Class:Error -> {error, {Class,Error}}
             end
+    end;
+connect(_Host, Port, UserOptions, NegotiationTimeout) ->
+    bad_arg([{port, Port},
+             {options, UserOptions},
+             {timeout, NegotiationTimeout}]).
+
+bad_arg(Args) ->
+    hd(bad_args(Args)).
+
+%% Special handling for finding the incorrect args for connect/3,
+%% which has two distinctly different signatures.
+bad_arg(Arg2, Arg3) ->
+    E0 = bad_args([{port, Arg2}, {options, Arg3}]),
+    E1 = bad_args([{options, Arg2}, {timeout, Arg3}]),
+    %% Select the case with only one error
+    case {E0, E1} of
+        {[Error], _}    -> Error;
+        {_, [Error]}    -> Error;
+        {[Error, _], _} -> Error
     end.
 
+%% Return list of errors
+-spec bad_args([{'options' | 'port' | 'timeout', any()}]) ->
+          [{'error', term()}].
+bad_args(Args) ->
+    IsErr = fun(true, _) -> false;
+               (false, Error) -> {true, {error, Error}}
+            end,
+    Check =
+        fun({options, Arg}) -> IsErr(?IS_VALID_OPTIONS(Arg), invalid_options);
+           ({timeout, Arg}) -> IsErr(?IS_VALID_TIMEOUT(Arg), invalid_timeout);
+           ({port, Arg})    -> IsErr(?IS_VALID_PORT(Arg), invalid_port)
+        end,
+
+    lists:filtermap(Check, Args).
+
 %%%----------------
 continue_connect(Socket, Options0, NegTimeout) ->
     {ok, {SockHost,SockPort}} = inet:sockname(Socket),
@@ -198,9 +412,10 @@ continue_connect(Socket, Options0, NegTimeout) ->
                        port = SockPort,
                        profile = ?GET_OPT(profile,Options)
                       },
-    ssh_system_sup:start_subsystem(client, Address, Socket, Options).
+    ssh_system_sup:start_connection(client, Address, Socket, Options).
 
 %%--------------------------------------------------------------------
+-doc "Closes an SSH connection.".
 -spec close(ConnectionRef) -> ok | {error,term()} when
       ConnectionRef :: connection_ref() .
 %%
@@ -212,9 +427,13 @@ close(ConnectionRef) ->
 %%--------------------------------------------------------------------
 %% Description: Retrieves information about a connection.
 %%---------------------------------------------------------------------
+-doc(#{title => <<"Other data types">>}).
 -type version() :: {protocol_version(), software_version()}.
+-doc(#{title => <<"Other data types">>}).
 -type protocol_version() :: {Major::pos_integer(), Minor::non_neg_integer()}.
+-doc(#{title => <<"Other data types">>}).
 -type software_version() :: string().
+-doc(#{title => <<"Other data types">>}).
 -type conn_info_algs() :: [{kex, kex_alg()}
                            | {hkey, pubkey_alg()}
                            | {encrypt, cipher_alg()}
@@ -226,8 +445,16 @@ close(ConnectionRef) ->
                            | {send_ext_info, boolean()}
                            | {recv_ext_info, boolean()}
                           ].
+-doc """
+Return values from the `connection_info/1` and `connection_info/2` functions.
+
+In the `option` info tuple are only the options included that differs from the
+default values.
+""".
+-doc(#{title => <<"Other data types">>}).
 -type conn_info_channels() :: [proplists:proplist()].
 
+-doc(#{title => <<"Other data types">>}).
 -type connection_info_tuple() ::
         {client_version, version()}
       | {server_version, version()}
@@ -237,16 +464,25 @@ close(ConnectionRef) ->
       | {options, client_options()}
       | {algorithms, conn_info_algs()}
       | {channels, conn_info_channels()}.
-        
--spec connection_info(ConnectionRef) -> InfoTupleList when
+
+-doc(#{equiv => connection_info/2}).
+-doc(#{since => <<"OTP 22.1">>}).
+-spec connection_info(ConnectionRef) ->
+          InfoTupleList | {error, term()} when
       ConnectionRef :: connection_ref(),
       InfoTupleList :: [InfoTuple],
       InfoTuple :: connection_info_tuple().
 
-connection_info(ConnectionRef) ->                                      
+connection_info(ConnectionRef) ->
     connection_info(ConnectionRef, []).
 
--spec connection_info(ConnectionRef, ItemList|Item) ->  InfoTupleList|InfoTuple when
+-doc """
+Returns information about a connection intended for e.g debugging or logging.
+
+When the `Key` is a single `Item`, the result is a single `InfoTuple`
+""".
+-spec connection_info(ConnectionRef, ItemList|Item) ->
+          InfoTupleList | InfoTuple | {error, term()} when
       ConnectionRef :: connection_ref(),
       ItemList :: [Item],
       Item :: client_version | server_version | user | peer | sockname | options | algorithms | sockname,
@@ -257,6 +493,7 @@ connection_info(ConnectionRef, Key) ->
     ssh_connection_handler:connection_info(ConnectionRef, Key).
 
 %%--------------------------------------------------------------------
+-doc false.
 -spec channel_info(connection_ref(), channel_id(), [atom()]) -> proplists:proplist().
 %%
 %% Description: Retrieves information about a connection.
@@ -268,12 +505,14 @@ channel_info(ConnectionRef, ChannelId, Options) ->
 %% Description: Starts a server listening for SSH connections
 %% on the given port.
 %%--------------------------------------------------------------------
+-doc(#{equiv => daemon/3}).
 -spec daemon(inet:port_number()) ->  {ok,daemon_ref()} | {error,term()}.
 
 daemon(Port) ->
     daemon(Port, []).
 
 
+-doc(#{equiv => daemon/3}).
 -spec daemon(inet:port_number()|open_socket(), daemon_options()) -> {ok,daemon_ref()} | {error,term()}.
 
 daemon(Port, UserOptions) when 0 =< Port,Port =< 65535 ->
@@ -293,7 +532,7 @@ daemon(Socket, UserOptions) ->
                                            profile = ?GET_OPT(profile,Options0)
                                           },
                         Options = ?PUT_INTERNAL_OPT({connected_socket, Socket}, Options0),
-                        case ssh_system_sup:start_subsystem(server, Address, Socket, Options) of
+                        case ssh_system_sup:start_connection(server, Address, Socket, Options) of
                             {ok,Pid} ->
                                 {ok,Pid};
                             {error, {already_started, _}} ->
@@ -320,6 +559,35 @@ daemon(Socket, UserOptions) ->
 
 
 
+-doc """
+daemon(HostAddress, Port, Options)
+
+Starts a server listening for SSH connections on the given port. If the `Port`
+is 0, a random free port is selected. See `daemon_info/1` about how to find the
+selected port number.
+
+As an alternative, an already open TCP socket could be passed to the function in
+`TcpSocket`. The SSH initiation and negotiation will be initiated on that one
+when an SSH starts at the other end of the TCP socket.
+
+For a description of the options, see [Daemon Options](`t:daemon_options/0`).
+
+Please note that by historical reasons both the `HostAddress` argument and the
+[gen_tcp connect_option() `{ip,Address}`](`t:gen_tcp:connect_option/0`) set the
+listening address. This is a source of possible inconsistent settings.
+
+The rules for handling the two address passing options are:
+
+- if `HostAddress` is an IP-address, that IP-address is the listening address.
+  An 'ip'-option will be discarded if present.
+- if `HostAddress` is the atom `loopback`, the listening address is `loopback`
+  and an loopback address will be chosen by the underlying layers. An
+  'ip'-option will be discarded if present.
+- if `HostAddress` is the atom `any` and no 'ip'-option is present, the
+  listening address is `any` and the socket will listen to all addresses
+- if `HostAddress` is `any` and an 'ip'-option is present, the listening address
+  is set to the value of the 'ip'-option
+""".
 -spec daemon(any | inet:ip_address(), inet:port_number(), daemon_options()) -> {ok,daemon_ref()} | {error,term()}
            ;(socket, open_socket(), daemon_options()) -> {ok,daemon_ref()} | {error,term()}
             .
@@ -342,8 +610,7 @@ daemon(Host0, Port0, UserOptions0) when 0 =< Port0, Port0 =< 65535,
 
                 %% throws error:Error if no usable hostkey is found
                 ssh_connection_handler:available_hkey_algorithms(server, Options1),
-                ssh_system_sup:start_system(server,
-                                            #address{address = Host,
+                ssh_system_sup:start_system(#address{address = Host,
                                                      port = Port,
                                                      profile = ?GET_OPT(profile,Options1)},
                                             Options1)
@@ -394,12 +661,47 @@ daemon(_, _, _) ->
     {error, badarg}.
 
 %%--------------------------------------------------------------------
+-doc """
+Replaces the options in a running daemon with the options in `NewUserOptions`.
+Only connections established after this call are affected, already established
+connections are not.
+
+> #### Note {: .info }
+>
+> In the final phase of this function, the listening process is restarted.
+> Therfore a connection attempt to the daemon in this final phase could fail.
+
+The handling of Erlang configurations is described in the User's Guide; see
+chapters [Configuration in SSH](configurations.md) and
+[Configuring algorithms in SSH](configure_algos.md).
+""".
+-doc(#{since => <<"OTP 25.1">>}).
+-spec daemon_replace_options(DaemonRef, NewUserOptions) -> {ok,daemon_ref()}
+                                                         | {error,term()} when
+      DaemonRef :: daemon_ref(),
+      NewUserOptions :: daemon_options().
+
+daemon_replace_options(DaemonRef, NewUserOptions) ->
+    {ok,Os0} = ssh_system_sup:get_acceptor_options(DaemonRef),
+    Os1 = ssh_options:merge_options(server, NewUserOptions, Os0),
+    ssh_system_sup:replace_acceptor_options(DaemonRef, Os1).
+
+%%--------------------------------------------------------------------
+-doc """
+Return values from the `daemon_info/1` and `daemon_info/2` functions.
+
+In the `option` info tuple are only the options included that differs from the
+default values.
+""".
+-doc(#{title => <<"Other data types">>}).
 -type daemon_info_tuple() ::
         {port, inet:port_number()}
       | {ip, inet:ip_address()}
       | {profile, atom()}
       | {options, daemon_options()}.
 
+-doc(#{equiv => daemon_info/2}).
+-doc(#{since => <<"OTP 19.0">>}).
 -spec daemon_info(DaemonRef) -> {ok,InfoTupleList} | {error,bad_daemon_ref} when
       DaemonRef :: daemon_ref(),
       InfoTupleList :: [InfoTuple],
@@ -436,6 +738,16 @@ daemon_info(DaemonRef) ->
 	    {error,bad_daemon_ref}
     end.
 
+-doc """
+Returns information about a daemon intended for e.g debugging or logging.
+
+When the `Key` is a single `Item`, the result is a single `InfoTuple`
+
+Note that [`daemon_info/1`](`daemon_info/1`) and
+[`daemon_info/2`](`daemon_info/2`) returns different types due to compatibility
+reasons.
+""".
+-doc(#{since => <<"OTP 22.1">>}).
 -spec daemon_info(DaemonRef, ItemList|Item) ->  InfoTupleList|InfoTuple | {error,bad_daemon_ref} when
       DaemonRef :: daemon_ref(),
       ItemList :: [Item],
@@ -461,53 +773,58 @@ daemon_info(DaemonRef, Keys) ->
 %% Description: Stops the listener, but leaves
 %% existing connections started by the listener up and running.
 %%--------------------------------------------------------------------
+-doc(#{equiv => stop_listener/3}).
 -spec stop_listener(daemon_ref()) -> ok.
 
 stop_listener(SysSup) ->
     ssh_system_sup:stop_listener(SysSup).
 
 
+-doc(#{equiv => stop_listener/3}).
 -spec stop_listener(inet:ip_address(), inet:port_number()) -> ok.
 
 stop_listener(Address, Port) ->
     stop_listener(Address, Port, ?DEFAULT_PROFILE).
 
 
+-doc """
+Stops the listener, but leaves existing connections started by the listener
+operational.
+""".
+-doc(#{since => <<"OTP 21.0">>}).
 -spec stop_listener(any|inet:ip_address(), inet:port_number(), term()) -> ok.
 
 stop_listener(Address, Port, Profile) ->
     lists:foreach(fun({Sup,_Addr}) ->
                           stop_listener(Sup)
                   end,
-                  ssh_system_sup:addresses(server,
-                                           #address{address=Address,
+                  ssh_system_sup:addresses(#address{address=Address,
                                                     port=Port,
                                                     profile=Profile})).
 
-%%--------------------------------------------------------------------
-%% Description: Stops the listener and all connections started by
-%% the listener.
-%%--------------------------------------------------------------------
+-doc(#{equiv => stop_daemon/3}).
 -spec stop_daemon(DaemonRef::daemon_ref()) -> ok.
 
 stop_daemon(SysSup) ->
-    ssh_system_sup:stop_system(server, SysSup).
+    ssh_system_sup:stop_system(SysSup).
 
 
+-doc(#{equiv => stop_daemon/3}).
 -spec stop_daemon(inet:ip_address(), inet:port_number()) -> ok.
 
 stop_daemon(Address, Port) ->
     stop_daemon(Address, Port, ?DEFAULT_PROFILE).
 
 
+-doc "Stops the listener and all connections started by the listener.".
+-doc(#{since => <<"OTP 21.0">>}).
 -spec stop_daemon(any|inet:ip_address(), inet:port_number(), atom()) -> ok.
 
 stop_daemon(Address, Port, Profile) ->
     lists:foreach(fun({Sup,_Addr}) ->
                           stop_daemon(Sup)
                   end,
-                  ssh_system_sup:addresses(server,
-                                           #address{address=Address,
+                  ssh_system_sup:addresses(#address{address=Address,
                                                     port=Port,
                                                     profile=Profile})).
 
@@ -517,6 +834,7 @@ stop_daemon(Address, Port, Profile) ->
 %% and will not return until the remote shell is ended.(e.g. on
 %% exit from the shell)
 %%--------------------------------------------------------------------
+-doc(#{equiv => shell/3}).
 -spec shell(open_socket() | host() | connection_ref()) ->  _.
 
 shell(ConnectionRef) when is_pid(ConnectionRef) ->
@@ -552,6 +870,7 @@ shell(Dest) ->
 
 
 
+-doc(#{equiv => shell/3}).
 -spec shell(open_socket() | host(), client_options()) ->  _.
 
 shell(Dest, Options) ->
@@ -574,6 +893,20 @@ shell_socket(Socket, Options) ->
     
 
 
+-doc """
+Connects to an SSH server at `Host` and `Port` (defaults to 22) and starts an
+interactive shell on that remote host.
+
+As an alternative, an already open TCP socket could be passed to the function in
+`TcpSocket`. The SSH initiation and negotiation will be initiated on that one
+and finally a shell will be started on the host at the other end of the TCP
+socket.
+
+For a description of the options, see [Client Options](`t:client_options/0`).
+
+The function waits for user input, and does not return until the remote shell is
+ended (that is, exit from the shell).
+""".
 -spec shell(Host, Port, Options) -> _ when
       Host :: host(),
       Port :: inet:port_number(),
@@ -589,12 +922,21 @@ shell(Host, Port, Options) ->
     end.
 
 %%--------------------------------------------------------------------
+-doc """
+Returns a key-value list, where the keys are the different types of algorithms
+and the values are the algorithms themselves.
+
+See the [User's Guide](configure_algos.md#example_default_algorithms) for an
+example.
+""".
+-doc(#{since => <<"OTP 18.0">>}).
 -spec default_algorithms() -> algs_list() .
 %%--------------------------------------------------------------------
 default_algorithms() ->
     ssh_transport:default_algorithms().
 
 %%--------------------------------------------------------------------
+-doc false.
 -spec chk_algos_opts(client_options()|daemon_options()) -> internal_options() | {error,term()}.
 %%--------------------------------------------------------------------
 chk_algos_opts(Opts) ->
@@ -617,6 +959,31 @@ chk_algos_opts(Opts) ->
 
 
 %%--------------------------------------------------------------------
+-doc """
+Sets tcp socket options on the tcp-socket below an ssh connection.
+
+This function calls the `inet:setopts/2`, read that documentation and for
+`t:gen_tcp:option/0`.
+
+All gen_tcp socket options except
+
+- `active`
+- `deliver`
+- `mode` and
+- `packet`
+
+are allowed. The excluded options are reserved by the SSH application.
+
+> #### Warning {: .warning }
+>
+> This is an extremely dangerous function. You use it on your own risk.
+>
+> Some options are OS and OS version dependent. Do not use it unless you know
+> what effect your option values will have on an TCP stream.
+>
+> Some values may destroy the functionality of the SSH protocol.
+""".
+-doc(#{since => <<"OTP 22.3">>}).
 -spec set_sock_opts(ConnectionRef, SocketOptions) ->
                            ok | {error, inet:posix()}  when
       ConnectionRef :: connection_ref(),
@@ -626,6 +993,12 @@ set_sock_opts(ConnectionRef, SocketOptions) ->
     ssh_connection_handler:set_sock_opts(ConnectionRef, SocketOptions).
 
 %%--------------------------------------------------------------------
+-doc """
+Get tcp socket option values of the tcp-socket below an ssh connection.
+
+This function calls the `inet:getopts/2`, read that documentation.
+""".
+-doc(#{since => <<"OTP 22.3">>}).
 -spec get_sock_opts(ConnectionRef, SocketGetOptions) ->
                            ok | {error, inet:posix()}  when
       ConnectionRef :: connection_ref(),
@@ -639,6 +1012,8 @@ get_sock_opts(ConnectionRef, SocketGetOptions) ->
 %% connects that address, connect to ConnectToHost:ConnectToPort from
 %% the server.
 %%--------------------------------------------------------------------
+-doc(#{equiv => tcpip_tunnel_to_server/6}).
+-doc(#{since => <<"OTP 23.0">>}).
 -spec tcpip_tunnel_to_server(ConnectionRef,
                              ListenHost, ListenPort,
                              ConnectToHost, ConnectToPort
@@ -655,6 +1030,21 @@ tcpip_tunnel_to_server(ConnectionHandler, ListenHost, ListenPort, ConnectToHost,
     tcpip_tunnel_to_server(ConnectionHandler, ListenHost, ListenPort, ConnectToHost, ConnectToPort, infinity).
 
 
+-doc """
+Tells the local client to listen to `ListenHost:ListenPort`. When someone
+connects to that address, the connection is forwarded in an encrypted channel to
+the peer server of `ConnectionRef`. That server then connects to
+`ConnectToHost:ConnectToPort`.
+
+The returned `TrueListenPort` is the port that is listened to. It is the same as
+`ListenPort`, except when `ListenPort = 0`. In that case a free port is selected
+by the underlying OS.
+
+Note that in case of an Erlang/OTP SSH server (daemon) as peer, that server must
+have been started with the option
+[tcpip_tunnel_in](`t:tcpip_tunnel_in_daemon_option/0`) to allow the connection.
+""".
+-doc(#{since => <<"OTP 23.0">>}).
 -spec tcpip_tunnel_to_server(ConnectionRef,
                              ListenHost, ListenPort,
                              ConnectToHost, ConnectToPort,
@@ -692,6 +1082,8 @@ tcpip_tunnel_to_server(ConnectionHandler, ListenHost, ListenPort, ConnectToHost0
 %% connects that address, connect to ConnectToHost:ConnectToPort from
 %% the client.
 %%--------------------------------------------------------------------
+-doc(#{equiv => tcpip_tunnel_from_server/6}).
+-doc(#{since => <<"OTP 23.0">>}).
 -spec tcpip_tunnel_from_server(ConnectionRef,
                                ListenHost, ListenPort,
                                ConnectToHost, ConnectToPort
@@ -707,6 +1099,22 @@ tcpip_tunnel_to_server(ConnectionHandler, ListenHost, ListenPort, ConnectToHost0
 tcpip_tunnel_from_server(ConnectionRef, ListenHost, ListenPort, ConnectToHost, ConnectToPort) ->
     tcpip_tunnel_from_server(ConnectionRef, ListenHost, ListenPort, ConnectToHost, ConnectToPort, infinity).
 
+-doc """
+Asks the remote server of `ConnectionRef` to listen to `ListenHost:ListenPort`.
+When someone connects that address, the connection is forwarded in an encrypted
+channel from the server to the client. The client (that is, at the node that
+calls this function) then connects to `ConnectToHost:ConnectToPort`.
+
+The returned `TrueListenPort` is the port that is listened to. It is the same as
+`ListenPort`, except when `ListenPort = 0`. In that case a free port is selected
+by the underlying OS.
+
+Note that in case of an Erlang/OTP SSH server (daemon) as peer, that server must
+have been started with the option
+[tcpip_tunnel_out](`t:tcpip_tunnel_out_daemon_option/0`) to allow the
+connection.
+""".
+-doc(#{since => <<"OTP 23.0">>}).
 -spec tcpip_tunnel_from_server(ConnectionRef,
                                ListenHost, ListenPort,
                                ConnectToHost, ConnectToPort,
@@ -744,11 +1152,44 @@ tcpip_tunnel_from_server(ConnectionRef, ListenHost0, ListenPort, ConnectToHost0,
 %%--------------------------------------------------------------------
 %% In move from public_key
 %%--------------------------------------------------------------------
+-doc(#{equiv => hostkey_fingerprint/2}).
+-doc(#{since => <<"OTP 24.0">>}).
 -spec hostkey_fingerprint(public_key:public_key()) -> string().
 
 hostkey_fingerprint(Key) ->
     sshfp_string(md5, ssh_message:ssh2_pubkey_encode(Key) ).
 
+-doc """
+hostkey_fingerprint([DigestType], HostKey) ->
+[string()]hostkey_fingerprint(DigestType, HostKey) -> string()
+
+Calculates a ssh fingerprint from a public host key as openssh does.
+
+The algorithm in [`hostkey_fingerprint/1`](`hostkey_fingerprint/1`) is md5 to be
+compatible with older ssh-keygen commands. The string from the second variant is
+prepended by the algorithm name in uppercase as in newer ssh-keygen commands.
+
+Examples:
+
+```erlang
+ 2> ssh:hostkey_fingerprint(Key).
+ "f5:64:a6:c1:5a:cb:9f:0a:10:46:a2:5c:3e:2f:57:84"
+
+ 3> ssh:hostkey_fingerprint(md5,Key).
+ "MD5:f5:64:a6:c1:5a:cb:9f:0a:10:46:a2:5c:3e:2f:57:84"
+
+ 4> ssh:hostkey_fingerprint(sha,Key).
+ "SHA1:bSLY/C4QXLDL/Iwmhyg0PGW9UbY"
+
+ 5> ssh:hostkey_fingerprint(sha256,Key).
+ "SHA256:aZGXhabfbf4oxglxltItWeHU7ub3Dc31NcNw2cMJePQ"
+
+ 6> ssh:hostkey_fingerprint([sha,sha256],Key).
+ ["SHA1:bSLY/C4QXLDL/Iwmhyg0PGW9UbY",
+  "SHA256:aZGXhabfbf4oxglxltItWeHU7ub3Dc31NcNw2cMJePQ"]
+```
+""".
+-doc(#{since => <<"OTP 24.0">>}).
 -spec hostkey_fingerprint(TypeOrTypes, Key) -> StringOrString
                                                    when
       TypeOrTypes :: public_key:digest_type() | [public_key:digest_type()],
@@ -787,7 +1228,7 @@ fp_fmt(b64, Bin) ->
     %%    [C || C<-base64:encode_to_string(Bin), C =/= $=]
     %% but I am not sure. Must be checked.
     B64Chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",
-    BitsInLast = 8*size(Bin) rem 6,
+    BitsInLast = 8*byte_size(Bin) rem 6,
     Padding = (6-BitsInLast) rem 6, % Want BitsInLast = [1:5] to map to padding [5:1] and 0 -> 0
     [lists:nth(C+1,B64Chars) || <<C:6>> <= <<Bin/binary,0:Padding>> ].
 
@@ -874,6 +1315,7 @@ transport_connect(Host, Port, SocketOpts, Options) ->
     Callback:connect(Host, Port, SocketOpts, ?GET_OPT(connect_timeout,Options)).
     
 %%%----------------------------------------------------------------
+-doc false.
 is_host(X, Opts) ->
     try is_host1(mangle_connect_address(X, Opts))
     catch
@@ -882,10 +1324,10 @@ is_host(X, Opts) ->
             
 
 is_host1(L) when is_list(L) -> true; %% "string()"
-is_host1(T) when is_tuple(T), size(T)==4 -> lists:all(fun(I) -> 0=<I andalso I=<255 end,
-                                                      tuple_to_list(T));
-is_host1(T) when is_tuple(T), size(T)==16 -> lists:all(fun(I) -> 0=<I andalso I=<65535 end,
-                                                       tuple_to_list(T));
+is_host1(T) when tuple_size(T)==4 -> lists:all(fun(I) -> 0=<I andalso I=<255 end,
+                                               tuple_to_list(T));
+is_host1(T) when tuple_size(T)==16 -> lists:all(fun(I) -> 0=<I andalso I=<65535 end,
+                                                tuple_to_list(T));
 is_host1(loopback) -> true.
 
 %%%----------------------------------------------------------------
@@ -928,18 +1370,23 @@ mangle_tunnel_address(X) when is_list(X) -> case catch inet:parse_address(X) of
 %%%# Tracing
 %%%#
 
+-doc false.
 ssh_dbg_trace_points() -> [tcp].
 
+-doc false.
 ssh_dbg_flags(tcp) -> [c].
 
+-doc false.
 ssh_dbg_on(tcp) -> dbg:tpl(?MODULE, controlling_process, 3, x),
                    dbg:tpl(?MODULE, transport_connect, 4, x),
                    dbg:tpl(?MODULE, close_listen_socket, 2, x).
                    
+-doc false.
 ssh_dbg_off(tcp) ->dbg:ctpl(?MODULE, controlling_process, 3),
                    dbg:ctpl(?MODULE, transport_connect, 4),
                    dbg:ctpl(?MODULE, close_listen_socket, 2).
 
+-doc false.
 ssh_dbg_format(tcp, {call, {?MODULE,controlling_process, [ListenSocket, ReqPid, _Opts]}}) ->
     ["TCP socket transferred to\n",
      io_lib:format("Sock: ~p~n"
@@ -956,6 +1403,7 @@ ssh_dbg_format(tcp, {return_from, {?MODULE,close_listen_socket,2}, _Result}) ->
     skip.
 
 
+-doc false.
 ssh_dbg_format(tcp, {call, {?MODULE,transport_connect, [Host,Port,SockOpts,_Opts]}}, Stack) ->
     {skip, [{transport_connect,Host,Port,SockOpts}|Stack]};
 ssh_dbg_format(tcp, {return_from, {?MODULE,transport_connect,4}, {ok,Sock}},
